/*****************************
* ZeusCMD - OpenGL Base Code *
* By Grant James (ZEUS)      *
* http://www.zeuscmd.com     *
*****************************/
#ifndef GLUTIL_H

#define GLUTIL_H

#pragma comment(lib, "opengl32.lib")
#pragma comment(lib, "glu32.lib")

#include <windows.h>
#include <gl/gl.h>
#include <gl/glu.h>
#include <stdio.h>

class OpenGL
{

public :

	static OpenGL *create()
	{ return instance = new OpenGL; }
	~OpenGL() { if (instance) delete instance; }

	bool initGL(char *t, int w = 640, int h = 480, bool full = false, int bpp = 32);
	bool changeDisplaySettings(int w, int h, bool full, int bpp);
	void runMessageLoop();
	void endMessageLoop() const { finished = true; }

	void displayError(char *title);

	void setInitFunc(bool (*i)())
	{ init = i; }
	void setDisplayFunc(void (*d)())
	{ display = d; }
	void setIdleFunc(void (*i)())
	{ idle = i; }
	void setResizeFunc(void (*r)(int w, int h))
	{ resize = r; }

	HWND getHWND() const
	{ return hwnd; }

	bool isKeyDown(int key) const
	{ return keys[key]; }
	void keyUp(int key)
	{ keys[key] = false; }

	bool isMouseDownL() const
	{ return mouseDownL; }

	int getMouseX() const
	{ return mouseX; }
	int getMouseY() const
	{ return mouseY; }

	int getWidth() const { return width; }
	int getHeight() const { return height; }

private :

	OpenGL() {}

	void killGL();

	static LRESULT WndProc(HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam);

	static OpenGL *instance;

	static char *title;
	static int width;
	static int height;
	static int bits;
	static bool fullscreen;

	static bool active;
	static bool finished;
	static bool keys[256];
	static int mouseX;
	static int mouseY;
	static bool mouseDownL;

	static HDC hdc;
	static HWND hwnd;
	static HGLRC hrc;

	static bool (*init)();
	static void (*display)();
	static void (*idle)();
	static void (*resize)(int w, int h);
};

#endif
